/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
            S P E C I A L  F U N C T I O N  P R O T O T Y P E S 
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

double derf_();					/* ERROR FUNCTION           */
double derfc();					/* COMPLEMENTARY ERROR      */
double normal(double);				/* NORMAL DISTRIBUTION      */
double normal2(double);				/* NORMAL DISTRIBUTION      */
/*
double gser(double,double,double,double);
double gcf(double,double,double,double);       
double gammln(double);				LOG GAMMA FUNCTION        
double gammq(double,double);			INCOMPLETE GAMMA FUNCTION 
*/
